@@ -1,20 +1,20 @@
 /*
  * Copyright (C) 1996-2002,2012 Michael R. Elkins <me@mutt.org>
- * 
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */ 
+ */
 
 #if HAVE_CONFIG_H
 # include "config.h"
@@ -123,29 +123,29 @@ static COLOR_LINE *mutt_new_color_line (void)
   COLOR_LINE *p = safe_calloc (1, sizeof (COLOR_LINE));
 
   p->fg = p->bg = -1;
-  
+
   return (p);
 }
 
-static void mutt_free_color_line(COLOR_LINE **l, 
+static void mutt_free_color_line(COLOR_LINE **l,
 				 int free_colors)
 {
   COLOR_LINE *tmp;
- 
-  if(!l || !*l)
+
+  if (!l || !*l)
     return;
 
   tmp = *l;
 
 #ifdef HAVE_COLOR
-  if(free_colors && tmp->fg != -1 && tmp->bg != -1)
+  if (free_colors && tmp->fg != -1 && tmp->bg != -1)
     mutt_free_color(tmp->fg, tmp->bg);
 #endif
 
   /* we should really introduce a container
    * type for regular expressions.
    */
-  
+
   regfree(&tmp->rx);
   mutt_pattern_free(&tmp->color_pattern);
   FREE (&tmp->pattern);
@@ -194,7 +194,7 @@ static char *get_color_name (char *dest, size_t destlen, int val)
     case COLOR_WHITE:
       strfcpy (dest, missing[1], destlen);
       return dest;
-      
+
     case COLOR_DEFAULT:
       strfcpy (dest, missing[2], destlen);
       return dest;
@@ -222,7 +222,7 @@ int mutt_alloc_color (int fg, int bg)
 {
   COLOR_LIST *p = ColorList;
   int i;
-  
+
 #if defined (USE_SLANG_CURSES)
   char fgc[SHORT_STRING], bgc[SHORT_STRING];
 #endif
@@ -269,8 +269,8 @@ int mutt_alloc_color (int fg, int bg)
     SLtt_set_color (i, NULL, get_color_name (fgc, sizeof (fgc), fg), get_color_name (bgc, sizeof (bgc), bg));
   else
 #elif defined (HAVE_USE_DEFAULT_COLORS)
-  if (fg == COLOR_DEFAULT)
-    fg = -1;
+    if (fg == COLOR_DEFAULT)
+      fg = -1;
   if (bg == COLOR_DEFAULT)
     bg = -1;
 #endif
@@ -297,7 +297,7 @@ void mutt_free_color (int fg, int bg)
 
       UserColors--;
       dprint(1,(debugfile,"mutt_free_color(): Color pairs used so far: %d\n",
-                           UserColors));
+                UserColors));
 
       if (p == ColorList)
       {
@@ -331,13 +331,18 @@ static int
 parse_color_name (const char *s, int *col, int *attr, int is_fg, BUFFER *err)
 {
   char *eptr;
-  int is_bright = 0;
+  int is_bright = 0, is_light = 0;
 
   if (ascii_strncasecmp (s, "bright", 6) == 0)
   {
     is_bright = 1;
     s += 6;
   }
+  else if (ascii_strncasecmp (s, "light", 5) == 0)
+  {
+    is_light = 1;
+    s += 5;
+  }
 
   /* allow aliases for xterm color resources */
   if (ascii_strncasecmp (s, "color", 5) == 0)
@@ -357,21 +362,33 @@ parse_color_name (const char *s, int *col, int *attr, int is_fg, BUFFER *err)
     return (-1);
   }
 
-  if (is_bright)
+  if (is_bright || is_light)
   {
     if (is_fg)
     {
-      *attr |= A_BOLD;
-    }
-    else if (COLORS < 16)
-    {
-      /* A_BLINK turns the background color brite on some terms */
-      *attr |= A_BLINK;
+      if ((COLORS >= 16) && is_light)
+      {
+        if (*col >= 0 && *col <= 7)
+        {
+          /* Advance the color 0-7 by 8 to get the light version */
+          *col += 8;
+        }
+      }
+      else
+      {
+        *attr |= A_BOLD;
+      }
     }
     else
     {
-      /* Advance the color by 8 to get the bright version */
-      *col += 8;
+      if (COLORS >= 16)
+      {
+        if (*col >= 0 && *col <= 7)
+        {
+          /* Advance the color 0-7 by 8 to get the light version */
+          *col += 8;
+        }
+      }
     }
   }
 
@@ -385,9 +402,9 @@ parse_color_name (const char *s, int *col, int *attr, int is_fg, BUFFER *err)
  * 	  unmono  index pattern [pattern...]
  */
 
-static int 
-_mutt_parse_uncolor (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err, 
-			 short parse_uncolor);
+static int
+_mutt_parse_uncolor (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err,
+                     short parse_uncolor);
 
 
 #ifdef HAVE_COLOR
@@ -445,19 +462,19 @@ static int _mutt_parse_uncolor (BUFFER *buf, BUFFER *s, unsigned long data,
     return (-1);
   }
 
-  if(
+  if (
 #ifdef HAVE_COLOR
-     /* we're running without curses */
-     option (OPTNOCURSES) 
-     || /* we're parsing an uncolor command, and have no colors */
-     (parse_uncolor && !has_colors())
-     /* we're parsing an unmono command, and have colors */
-     || (!parse_uncolor && has_colors())
+    /* we're running without curses */
+    option (OPTNOCURSES)
+    || /* we're parsing an uncolor command, and have no colors */
+    (parse_uncolor && !has_colors())
+    /* we're parsing an unmono command, and have colors */
+    || (!parse_uncolor && has_colors())
 #else
-     /* We don't even have colors compiled in */
-     parse_uncolor
+    /* We don't even have colors compiled in */
+    parse_uncolor
 #endif
-     )
+    )
   {
     /* just eat the command, but don't do anything real about it */
     do
@@ -491,7 +508,7 @@ static int _mutt_parse_uncolor (BUFFER *buf, BUFFER *s, unsigned long data,
           if (!do_cache)
 	    do_cache = 1;
 	  dprint(1,(debugfile,"Freeing pattern \"%s\" from color list\n",
-	                       tmp->pattern));
+                    tmp->pattern));
 	  if (last)
 	    last->next = tmp->next;
 	  else
@@ -517,14 +534,14 @@ static int _mutt_parse_uncolor (BUFFER *buf, BUFFER *s, unsigned long data,
 }
 
 
-static int 
+static int
 add_pattern (COLOR_LINE **top, const char *s, int sensitive,
 	     int fg, int bg, int attr, BUFFER *err,
 	     int is_index)
 {
 
   /* is_index used to store compiled pattern
-   * only for `index' color object 
+   * only for `index' color object
    * when called from mutt_parse_color() */
 
   COLOR_LINE *tmp = *top;
@@ -565,14 +582,17 @@ add_pattern (COLOR_LINE **top, const char *s, int sensitive,
   else
   {
     int r;
-    char buf[LONG_STRING];
+    BUFFER *buf = NULL;
 
     tmp = mutt_new_color_line ();
-    if (is_index) 
+    if (is_index)
     {
-      strfcpy(buf, NONULL(s), sizeof(buf));
-      mutt_check_simple (buf, sizeof (buf), NONULL(SimpleSearch));
-      if((tmp->color_pattern = mutt_pattern_comp (buf, MUTT_FULL_MSG, err)) == NULL)
+      buf = mutt_buffer_pool_get ();
+      mutt_buffer_strcpy(buf, NONULL(s));
+      mutt_check_simple (buf, NONULL(SimpleSearch));
+      tmp->color_pattern = mutt_pattern_comp (buf->data, MUTT_FULL_MSG, err);
+      mutt_buffer_pool_release (&buf);
+      if (tmp->color_pattern == NULL)
       {
 	mutt_free_color_line(&tmp, 1);
 	return -1;
@@ -587,7 +607,7 @@ add_pattern (COLOR_LINE **top, const char *s, int sensitive,
     tmp->next = *top;
     tmp->pattern = safe_strdup (s);
 #ifdef HAVE_COLOR
-    if(fg != -1 && bg != -1)
+    if (fg != -1 && bg != -1)
     {
       tmp->fg = fg;
       tmp->bg = bg;
@@ -615,20 +635,20 @@ parse_object(BUFFER *buf, BUFFER *s, int *o, int *ql, BUFFER *err)
 {
   int q_level = 0;
   char *eptr;
-  
-  if(!MoreArgs(s))
+
+  if (!MoreArgs(s))
   {
     strfcpy(err->data, _("Missing arguments."), err->dsize);
     return -1;
   }
-  
+
   mutt_extract_token(buf, s, 0);
-  if(!mutt_strncmp(buf->data, "quoted", 6))
+  if (!mutt_strncmp(buf->data, "quoted", 6))
   {
-    if(buf->data[6])
+    if (buf->data[6])
     {
       *ql = strtol(buf->data + 6, &eptr, 10);
-      if(*eptr || q_level < 0)
+      if (*eptr || q_level < 0)
       {
 	snprintf(err->data, err->dsize, _("%s: no such object"), buf->data);
 	return -1;
@@ -636,7 +656,7 @@ parse_object(BUFFER *buf, BUFFER *s, int *o, int *ql, BUFFER *err)
     }
     else
       *ql = 0;
-    
+
     *o = MT_COLOR_QUOTED;
   }
   else if (!ascii_strcasecmp(buf->data, "compose"))
@@ -671,6 +691,36 @@ typedef int (*parser_callback_t)(BUFFER *, BUFFER *, int *, int *, int *, BUFFER
 static int
 parse_color_pair(BUFFER *buf, BUFFER *s, int *fg, int *bg, int *attr, BUFFER *err)
 {
+  FOREVER
+  {
+    if (! MoreArgs (s))
+    {
+      strfcpy (err->data, _("color: too few arguments"), err->dsize);
+      return (-1);
+    }
+
+    mutt_extract_token (buf, s, 0);
+
+    if (ascii_strcasecmp ("bold", buf->data) == 0)
+      *attr |= A_BOLD;
+    else if (ascii_strcasecmp ("underline", buf->data) == 0)
+      *attr |= A_UNDERLINE;
+    else if (ascii_strcasecmp ("none", buf->data) == 0)
+      *attr = A_NORMAL;
+    else if (ascii_strcasecmp ("reverse", buf->data) == 0)
+      *attr |= A_REVERSE;
+    else if (ascii_strcasecmp ("standout", buf->data) == 0)
+      *attr |= A_STANDOUT;
+    else if (ascii_strcasecmp ("normal", buf->data) == 0)
+      *attr = A_NORMAL; /* needs use = instead of |= to clear other bits */
+    else
+    {
+      if (parse_color_name (buf->data, fg, attr, 1, err) != 0)
+        return (-1);
+      break;
+    }
+  }
+
   if (! MoreArgs (s))
   {
     strfcpy (err->data, _("color: too few arguments"), err->dsize);
@@ -679,20 +729,9 @@ parse_color_pair(BUFFER *buf, BUFFER *s, int *fg, int *bg, int *attr, BUFFER *er
 
   mutt_extract_token (buf, s, 0);
 
-  if (parse_color_name (buf->data, fg, attr, 1, err) != 0)
-    return (-1);
-
-  if (! MoreArgs (s))
-  {
-    strfcpy (err->data, _("color: too few arguments"), err->dsize);
-    return (-1);
-  }
-  
-  mutt_extract_token (buf, s, 0);
-
   if (parse_color_name (buf->data, bg, attr, 0, err) != 0)
     return (-1);
-  
+
   return 0;
 }
 
@@ -701,9 +740,9 @@ parse_color_pair(BUFFER *buf, BUFFER *s, int *fg, int *bg, int *attr, BUFFER *er
 static int
 parse_attr_spec(BUFFER *buf, BUFFER *s, int *fg, int *bg, int *attr, BUFFER *err)
 {
-  
-  if(fg) *fg = -1; 
-  if(bg) *bg = -1;
+
+  if (fg) *fg = -1;
+  if (bg) *bg = -1;
 
   if (! MoreArgs (s))
   {
@@ -730,39 +769,39 @@ parse_attr_spec(BUFFER *buf, BUFFER *s, int *fg, int *bg, int *attr, BUFFER *err
     snprintf (err->data, err->dsize, _("%s: no such attribute"), buf->data);
     return (-1);
   }
-  
+
   return 0;
 }
 
 static int fgbgattr_to_color(int fg, int bg, int attr)
 {
 #ifdef HAVE_COLOR
-  if(fg != -1 && bg != -1)
+  if (fg != -1 && bg != -1)
     return attr | mutt_alloc_color(fg, bg);
   else
 #endif
     return attr;
 }
 
-/* usage: color <object> <fg> <bg> [ <regexp> ] 
+/* usage: color <object> <fg> <bg> [ <regexp> ]
  * 	  mono  <object> <attr> [ <regexp> ]
  */
 
-static int 
-_mutt_parse_color (BUFFER *buf, BUFFER *s, BUFFER *err, 
+static int
+_mutt_parse_color (BUFFER *buf, BUFFER *s, BUFFER *err,
 		   parser_callback_t callback, short dry_run)
 {
   int object = 0, attr = 0, fg = 0, bg = 0, q_level = 0;
   int r = 0;
 
-  if(parse_object(buf, s, &object, &q_level, err) == -1)
+  if (parse_object(buf, s, &object, &q_level, err) == -1)
     return -1;
 
-  if(callback(buf, s, &fg, &bg, &attr, err) == -1)
+  if (callback(buf, s, &fg, &bg, &attr, err) == -1)
     return -1;
 
   /* extract a regular expression if needed */
-  
+
   if (object == MT_COLOR_HEADER || object == MT_COLOR_BODY || object == MT_COLOR_INDEX)
   {
     if (!MoreArgs (s))
@@ -773,31 +812,31 @@ _mutt_parse_color (BUFFER *buf, BUFFER *s, BUFFER *err,
 
     mutt_extract_token (buf, s, 0);
   }
-   
+
   if (MoreArgs (s))
   {
     strfcpy (err->data, _("too many arguments"), err->dsize);
     return (-1);
   }
-  
-  /* dry run? */
-  
-  if(dry_run) return 0;
 
-  
+  /* dry run? */
+
+  if (dry_run) return 0;
+
+
 #ifdef HAVE_COLOR
 # ifdef HAVE_USE_DEFAULT_COLORS
   if (!option (OPTNOCURSES) && has_colors()
-    /* delay use_default_colors() until needed, since it initializes things */
-    && (fg == COLOR_DEFAULT || bg == COLOR_DEFAULT)
-    && use_default_colors () != OK)
+      /* delay use_default_colors() until needed, since it initializes things */
+      && (fg == COLOR_DEFAULT || bg == COLOR_DEFAULT)
+      && use_default_colors () != OK)
   {
     strfcpy (err->data, _("default colors not supported"), err->dsize);
     return (-1);
   }
 # endif /* HAVE_USE_DEFAULT_COLORS */
 #endif
-  
+
   if (object == MT_COLOR_HEADER)
     r = add_pattern (&ColorHdrList, buf->data, 0, fg, bg, attr, err,0);
   else if (object == MT_COLOR_BODY)
@@ -820,7 +859,7 @@ _mutt_parse_color (BUFFER *buf, BUFFER *s, BUFFER *err,
     if (q_level == 0)
     {
       ColorDefs[MT_COLOR_QUOTED] = fgbgattr_to_color(fg, bg, attr);
-      
+
       ColorQuote[0] = ColorDefs[MT_COLOR_QUOTED];
       for (q_level = 1; q_level < ColorQuoteUsed; q_level++)
       {
@@ -842,10 +881,10 @@ _mutt_parse_color (BUFFER *buf, BUFFER *s, BUFFER *err,
 int mutt_parse_color(BUFFER *buff, BUFFER *s, unsigned long data, BUFFER *err)
 {
   int dry_run = 0;
-  
-  if(option(OPTNOCURSES) || !has_colors())
+
+  if (option(OPTNOCURSES) || !has_colors())
     dry_run = 1;
-  
+
   return _mutt_parse_color(buff, s, err, parse_color_pair, dry_run);
 }
 
@@ -854,15 +893,14 @@ int mutt_parse_color(BUFFER *buff, BUFFER *s, unsigned long data, BUFFER *err)
 int mutt_parse_mono(BUFFER *buff, BUFFER *s, unsigned long data, BUFFER *err)
 {
   int dry_run = 0;
-  
+
 #ifdef HAVE_COLOR
-  if(option(OPTNOCURSES) || has_colors())
+  if (option(OPTNOCURSES) || has_colors())
     dry_run = 1;
 #else
-  if(option(OPTNOCURSES))
+  if (option(OPTNOCURSES))
     dry_run = 1;
 #endif
 
   return _mutt_parse_color(buff, s, err, parse_attr_spec, dry_run);
 }
-
