@@ -114,6 +114,9 @@ void mutt_read_histfile (void)
   char *linebuf = NULL, *p;
   size_t buflen;
 
+  if (!HistFile)
+    return;
+
   if ((f = fopen (HistFile, "r")) == NULL)
     return;
 
@@ -401,7 +404,7 @@ void mutt_history_add (history_class_t hclass, const char *s, int save)
     {
       if (option (OPTHISTREMOVEDUPS))
         remove_history_dups (hclass, s);
-      if (save && SaveHist)
+      if (save && SaveHist && HistFile)
         save_history (hclass, s);
       mutt_str_replace (&h->hist[h->last++], s);
       if (h->last > HistSize)
