@@ -67,7 +67,7 @@ static FILE* msg_cache_get (IMAP_DATA* idata, HEADER* h);
 static FILE* msg_cache_put (IMAP_DATA* idata, HEADER* h);
 static int msg_cache_commit (IMAP_DATA* idata, HEADER* h);
 
-static void flush_buffer(char* buf, size_t* len, CONNECTION* conn);
+static int flush_buffer (char* buf, size_t* len, CONNECTION* conn);
 static int msg_fetch_header (CONTEXT* ctx, IMAP_HEADER* h, char* buf,
                              FILE* fp);
 static int msg_parse_fetch (IMAP_HEADER* h, char* s);
@@ -1130,7 +1130,7 @@ int imap_commit_message (CONTEXT *ctx, MESSAGE *msg)
 int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
 {
   IMAP_DATA* idata;
-  FILE *fp;
+  FILE *fp = NULL;
   char buf[LONG_STRING*2];
   char mbox[LONG_STRING];
   char mailbox[LONG_STRING];
@@ -1201,20 +1201,7 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
   while (rc == IMAP_CMD_CONTINUE);
 
   if (rc != IMAP_CMD_RESPOND)
-  {
-    char *pc;
-
-    dprint (1, (debugfile, "imap_append_message(): command failed: %s\n",
-		idata->buf));
-
-    pc = idata->buf + SEQLEN;
-    SKIPWS (pc);
-    pc = imap_next_word (pc);
-    mutt_error ("%s", pc);
-    mutt_sleep (1);
-    safe_fclose (&fp);
-    goto fail;
-  }
+    goto cmd_step_fail;
 
   for (last = EOF, sent = len = 0; (c = fgetc(fp)) != EOF; last = c)
   {
@@ -1226,39 +1213,48 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
     if (len > sizeof(buf) - 3)
     {
       sent += len;
-      flush_buffer(buf, &len, idata->conn);
+      if (flush_buffer (buf, &len, idata->conn) < 0)
+        goto fail;
       mutt_progress_update (&progressbar, sent, -1);
     }
   }
 
   if (len)
-    flush_buffer(buf, &len, idata->conn);
+    if (flush_buffer (buf, &len, idata->conn) < 0)
+      goto fail;
 
-  mutt_socket_write (idata->conn, "\r\n");
+  if (mutt_socket_write (idata->conn, "\r\n") < 0)
+    goto fail;
   safe_fclose (&fp);
 
   do
     rc = imap_cmd_step (idata);
   while (rc == IMAP_CMD_CONTINUE);
 
-  if (!imap_code (idata->buf))
-  {
-    char *pc;
-
-    dprint (1, (debugfile, "imap_append_message(): command failed: %s\n",
-		idata->buf));
-    pc = idata->buf + SEQLEN;
-    SKIPWS (pc);
-    pc = imap_next_word (pc);
-    mutt_error ("%s", pc);
-    mutt_sleep (1);
-    goto fail;
-  }
+  if (rc != IMAP_CMD_OK)
+    goto cmd_step_fail;
 
   FREE (&mx.mbox);
   return 0;
 
+cmd_step_fail:
+  dprint (1, (debugfile, "imap_append_message(): command failed: %s\n",
+              idata->buf));
+  if (rc != IMAP_CMD_BAD)
+  {
+    char *pc;
+
+    pc = imap_next_word (idata->buf);  /* skip sequence number or token */
+    pc = imap_next_word (pc);          /* skip response code */
+    if (*pc)
+    {
+      mutt_error ("%s", pc);
+      mutt_sleep (1);
+    }
+  }
+
 fail:
+  safe_fclose (&fp);
   FREE (&mx.mbox);
   return -1;
 }
@@ -1912,9 +1908,13 @@ static char* msg_parse_flags (IMAP_HEADER* h, char* s)
   return s;
 }
 
-static void flush_buffer(char *buf, size_t *len, CONNECTION *conn)
+static int flush_buffer (char *buf, size_t *len, CONNECTION *conn)
 {
+  int rc;
+
   buf[*len] = '\0';
-  mutt_socket_write_n(conn, buf, *len);
+  rc = mutt_socket_write_n(conn, buf, *len);
   *len = 0;
+
+  return rc;
 }
