@@ -1,20 +1,20 @@
 /*
  * Copyright (C) 1996-2000,2012-2013 Michael R. Elkins <me@mutt.org>
- * 
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */ 
+ */
 
 #if HAVE_CONFIG_H
 # include "config.h"
@@ -48,7 +48,7 @@ char *mutt_read_rfc822_line (FILE *f, char *line, size_t *linelen)
   FOREVER
   {
     if (fgets (buf, *linelen - offset, f) == NULL ||	/* end of file or */
-	(ISSPACE (*line) && !offset))			/* end of headers */ 
+	(ISSPACE (*line) && !offset))			/* end of headers */
     {
       *line = 0;
       return (line);
@@ -144,7 +144,7 @@ static PARAMETER *parse_parameters (const char *s)
   size_t i;
 
   dprint (2, (debugfile, "parse_parameters: `%s'\n", s));
-  
+
   while (*s)
   {
     if ((p = strpbrk (s, "=;")) == NULL)
@@ -183,7 +183,8 @@ static PARAMETER *parse_parameters (const char *s)
 	s++;
 	for (i=0; *s && i < sizeof (buffer) - 1; i++, s++)
 	{
-	  if (AssumedCharset && *AssumedCharset) {
+	  if (AssumedCharset && *AssumedCharset)
+          {
             /* As iso-2022-* has a character of '"' with non-ascii state,
 	     * ignore it. */
             if (*s == 0x1b && i < sizeof (buffer) - 2)
@@ -223,8 +224,8 @@ static PARAMETER *parse_parameters (const char *s)
 	new->value = safe_strdup (buffer);
 
 	dprint (2, (debugfile, "parse_parameter: `%s' = `%s'\n",
-	      new->attribute ? new->attribute : "",
-	      new->value ? new->value : ""));
+                    new->attribute ? new->attribute : "",
+                    new->value ? new->value : ""));
 
 	/* Add this parameter to the list */
 	if (head)
@@ -252,7 +253,7 @@ static PARAMETER *parse_parameters (const char *s)
       s = skip_email_wsp(s + 1);
     }
     while (*s == ';'); /* skip empty parameters */
-  }    
+  }
 
 bail:
 
@@ -266,7 +267,7 @@ int mutt_check_mime_type (const char *s)
     return TYPETEXT;
   else if (ascii_strcasecmp ("multipart", s) == 0)
     return TYPEMULTIPART;
-#ifdef SUN_ATTACHMENT 
+#ifdef SUN_ATTACHMENT
   else if (ascii_strcasecmp ("x-sun-attachment", s) == 0)
     return TYPEMULTIPART;
 #endif
@@ -311,20 +312,20 @@ void mutt_parse_content_type (char *s, BODY *ct)
      * let that take precedence, and don't set it here */
     if ((pc = mutt_get_parameter( "name", ct->parameter)) && !ct->filename)
       ct->filename = safe_strdup(pc);
-    
+
 #ifdef SUN_ATTACHMENT
     /* this is deep and utter perversion */
     if ((pc = mutt_get_parameter ("conversions", ct->parameter)))
       ct->encoding = mutt_check_encoding (pc);
 #endif
-    
+
   }
-  
+
   /* Now get the subtype */
   if ((subtype = strchr(s, '/')))
   {
     *subtype++ = '\0';
-    for(pc = subtype; *pc && !ISSPACE(*pc) && *pc != ';'; pc++)
+    for (pc = subtype; *pc && !ISSPACE(*pc) && *pc != ';'; pc++)
       ;
     *pc = '\0';
     ct->subtype = safe_strdup (subtype);
@@ -335,7 +336,7 @@ void mutt_parse_content_type (char *s, BODY *ct)
 
 #ifdef SUN_ATTACHMENT
   if (ascii_strcasecmp ("x-sun-attachment", s) == 0)
-      ct->subtype = safe_strdup ("x-sun-attachment");
+    ct->subtype = safe_strdup ("x-sun-attachment");
 #endif
 
   if (ct->type == TYPEOTHER)
@@ -371,8 +372,8 @@ void mutt_parse_content_type (char *s, BODY *ct)
   {
     if (!(pc = mutt_get_parameter ("charset", ct->parameter)))
       mutt_set_parameter ("charset", (AssumedCharset && *AssumedCharset) ?
-                         (const char *) mutt_get_default_charset ()
-                         : "us-ascii", &ct->parameter);
+                          (const char *) mutt_get_default_charset ()
+                          : "us-ascii", &ct->parameter);
   }
 
 }
@@ -410,16 +411,17 @@ static void parse_content_disposition (const char *s, BODY *ct)
 BODY *mutt_read_mime_header (FILE *fp, int digest)
 {
   BODY *p = mutt_new_body();
+  ENVELOPE *e = mutt_new_envelope ();
   char *c;
   char *line = safe_malloc (LONG_STRING);
   size_t linelen = LONG_STRING;
-  
+
   p->hdr_offset  = ftello (fp);
 
   p->encoding    = ENC7BIT; /* default from RFC1521 */
   p->type        = digest ? TYPEMESSAGE : TYPETEXT;
   p->disposition = DISPINLINE;
-  
+
   while (*(line = mutt_read_rfc822_line (fp, line, &linelen)) != 0)
   {
     /* Find the value of the current header */
@@ -452,7 +454,7 @@ BODY *mutt_read_mime_header (FILE *fp, int digest)
 	mutt_str_replace (&p->description, c);
 	rfc2047_decode (&p->description);
       }
-    } 
+    }
 #ifdef SUN_ATTACHMENT
     else if (!ascii_strncasecmp ("x-sun-", line, 6))
     {
@@ -469,6 +471,11 @@ BODY *mutt_read_mime_header (FILE *fp, int digest)
       }
     }
 #endif
+    else
+    {
+      if (mutt_parse_rfc822_line (e, NULL, line, c, 0, 0, 0, NULL))
+        p->mime_headers = e;
+    }
   }
   p->offset = ftello (fp); /* Mark the start of the real data */
   if (p->type == TYPETEXT && !p->subtype)
@@ -478,6 +485,11 @@ BODY *mutt_read_mime_header (FILE *fp, int digest)
 
   FREE (&line);
 
+  if (p->mime_headers)
+    rfc2047_decode_envelope (p->mime_headers);
+  else
+    mutt_free_envelope (&e);
+
   return (p);
 }
 
@@ -490,13 +502,13 @@ void mutt_parse_part (FILE *fp, BODY *b)
     case TYPEMULTIPART:
 #ifdef SUN_ATTACHMENT
       if ( !ascii_strcasecmp (b->subtype, "x-sun-attachment") )
-          bound = "--------";
+        bound = "--------";
       else
 #endif
-          bound = mutt_get_parameter ("boundary", b->parameter);
+        bound = mutt_get_parameter ("boundary", b->parameter);
 
       fseeko (fp, b->offset, SEEK_SET);
-      b->parts =  mutt_parse_multipart (fp, bound, 
+      b->parts =  mutt_parse_multipart (fp, bound,
 					b->offset + b->length,
 					ascii_strcasecmp ("digest", b->subtype) == 0);
       break;
@@ -624,20 +636,21 @@ BODY *mutt_parse_multipart (FILE *fp, const char *boundary, LOFF_T end_off, int
 	new = mutt_read_mime_header (fp, digest);
 
 #ifdef SUN_ATTACHMENT
-        if (mutt_get_parameter ("content-lines", new->parameter)) {
+        if (mutt_get_parameter ("content-lines", new->parameter))
+        {
 	  mutt_atoi (mutt_get_parameter ("content-lines", new->parameter), &lines);
 	  for ( ; lines; lines-- )
-	     if (ftello (fp) >= end_off || fgets (buffer, LONG_STRING, fp) == NULL)
-	       break;
+            if (ftello (fp) >= end_off || fgets (buffer, LONG_STRING, fp) == NULL)
+              break;
 	}
 #endif
-	
+
 	/*
 	 * Consistency checking - catch
 	 * bad attachment end boundaries
 	 */
-	
-	if(new->offset > end_off)
+
+	if (new->offset > end_off)
 	{
 	  mutt_free_body(&new);
 	  break;
@@ -658,9 +671,9 @@ BODY *mutt_parse_multipart (FILE *fp, const char *boundary, LOFF_T end_off, int
     last->length = end_off - last->offset;
 
   /* parse recursive MIME parts */
-  for(last = head; last; last = last->next)
+  for (last = head; last; last = last->next)
     mutt_parse_part(fp, last);
-  
+
   return (head);
 }
 
@@ -764,11 +777,11 @@ time_t mutt_parse_date (const char *s, HEADER *h)
   char scratch[SHORT_STRING];
 
   /* Don't modify our argument. Fixed-size buffer is ok here since
-   * the date format imposes a natural limit. 
+   * the date format imposes a natural limit.
    */
 
   strfcpy (scratch, s, sizeof (scratch));
-  
+
   /* kill the day of the week, if it exists. */
   if ((t = strchr (scratch, ',')))
     t++;
@@ -910,7 +923,7 @@ char *mutt_extract_message_id (const char *s, const char **saveptr)
   {
     if (*p == '<')
     {
-      s = p; 
+      s = p;
       o = onull = NULL;
       continue;
     }
@@ -932,7 +945,7 @@ char *mutt_extract_message_id (const char *s, const char **saveptr)
     }
 
     /* some idiotic clients break their message-ids between lines */
-    if (s == p) 
+    if (s == p)
       /* step past another whitespace */
       s = p + 1;
     else if (o)
@@ -954,7 +967,8 @@ void mutt_parse_mime_message (CONTEXT *ctx, HEADER *cur)
 {
   MESSAGE *msg;
 
-  do {
+  do
+  {
     if (cur->content->type != TYPEMESSAGE &&
         cur->content->type != TYPEMULTIPART)
       break; /* nothing to do */
@@ -976,302 +990,339 @@ void mutt_parse_mime_message (CONTEXT *ctx, HEADER *cur)
   cur->attach_valid = 0;
 }
 
+void mutt_auto_subscribe (const char *mailto)
+{
+  ENVELOPE *lpenv;
+
+  if (!AutoSubscribeCache)
+    AutoSubscribeCache = hash_create (200, MUTT_HASH_STRCASECMP | MUTT_HASH_STRDUP_KEYS);
+
+  if (!mailto || hash_find (AutoSubscribeCache, mailto))
+    return;
+
+  hash_insert (AutoSubscribeCache, mailto, AutoSubscribeCache);
+
+  lpenv = mutt_new_envelope (); /* parsed envelope from the List-Post mailto: URL */
+
+  if ((url_parse_mailto (lpenv, NULL, mailto) != -1) &&
+      lpenv->to && lpenv->to->mailbox &&
+      !mutt_match_rx_list (lpenv->to->mailbox, SubscribedLists) &&
+      !mutt_match_rx_list (lpenv->to->mailbox, UnMailLists) &&
+      !mutt_match_rx_list (lpenv->to->mailbox, UnSubscribedLists))
+  {
+    BUFFER err;
+    char errbuf[STRING];
+
+    memset (&err, 0, sizeof(err));
+    err.data = errbuf;
+    err.dsize = sizeof(errbuf);
+
+    /* mutt_add_to_rx_list() detects duplicates, so it is safe to
+     * try to add here without any checks. */
+    mutt_add_to_rx_list (&MailLists, lpenv->to->mailbox, REG_ICASE, &err);
+    mutt_add_to_rx_list (&SubscribedLists, lpenv->to->mailbox, REG_ICASE, &err);
+  }
+  mutt_free_envelope (&lpenv);
+}
+
 int mutt_parse_rfc822_line (ENVELOPE *e, HEADER *hdr, char *line, char *p, short user_hdrs, short weed,
 			    short do_2047, LIST **lastp)
 {
   int matched = 0;
-  
+
   switch (ascii_tolower (line[0]))
   {
     case 'a':
-    if (ascii_strcasecmp (line+1, "pparently-to") == 0)
-    {
-      e->to = rfc822_parse_adrlist (e->to, p);
-      matched = 1;
-    }
-    else if (ascii_strcasecmp (line+1, "pparently-from") == 0)
-    {
-      e->from = rfc822_parse_adrlist (e->from, p);
-      matched = 1;
-    }
-    break;
-    
+      if (ascii_strcasecmp (line+1, "pparently-to") == 0)
+      {
+        e->to = rfc822_parse_adrlist (e->to, p);
+        matched = 1;
+      }
+      else if (ascii_strcasecmp (line+1, "pparently-from") == 0)
+      {
+        e->from = rfc822_parse_adrlist (e->from, p);
+        matched = 1;
+      }
+      break;
+
     case 'b':
-    if (ascii_strcasecmp (line+1, "cc") == 0)
-    {
-      e->bcc = rfc822_parse_adrlist (e->bcc, p);
-      matched = 1;
-    }
-    break;
-    
+      if (ascii_strcasecmp (line+1, "cc") == 0)
+      {
+        e->bcc = rfc822_parse_adrlist (e->bcc, p);
+        matched = 1;
+      }
+      break;
+
     case 'c':
-    if (ascii_strcasecmp (line+1, "c") == 0)
-    {
-      e->cc = rfc822_parse_adrlist (e->cc, p);
-      matched = 1;
-    }
-    else if (ascii_strncasecmp (line + 1, "ontent-", 7) == 0)
-    {
-      if (ascii_strcasecmp (line+8, "type") == 0)
+      if (ascii_strcasecmp (line+1, "c") == 0)
       {
-	if (hdr)
-	  mutt_parse_content_type (p, hdr->content);
-	matched = 1;
+        e->cc = rfc822_parse_adrlist (e->cc, p);
+        matched = 1;
       }
-      else if (ascii_strcasecmp (line+8, "transfer-encoding") == 0)
+      else if (ascii_strncasecmp (line + 1, "ontent-", 7) == 0)
       {
-	if (hdr)
-	  hdr->content->encoding = mutt_check_encoding (p);
-	matched = 1;
+        if (ascii_strcasecmp (line+8, "type") == 0)
+        {
+          if (hdr)
+            mutt_parse_content_type (p, hdr->content);
+          matched = 1;
+        }
+        else if (ascii_strcasecmp (line+8, "transfer-encoding") == 0)
+        {
+          if (hdr)
+            hdr->content->encoding = mutt_check_encoding (p);
+          matched = 1;
+        }
+        else if (ascii_strcasecmp (line+8, "length") == 0)
+        {
+          if (hdr)
+          {
+            if ((hdr->content->length = atol (p)) < 0)
+              hdr->content->length = -1;
+          }
+          matched = 1;
+        }
+        else if (ascii_strcasecmp (line+8, "description") == 0)
+        {
+          if (hdr)
+          {
+            mutt_str_replace (&hdr->content->description, p);
+            rfc2047_decode (&hdr->content->description);
+          }
+          matched = 1;
+        }
+        else if (ascii_strcasecmp (line+8, "disposition") == 0)
+        {
+          if (hdr)
+            parse_content_disposition (p, hdr->content);
+          matched = 1;
+        }
       }
-      else if (ascii_strcasecmp (line+8, "length") == 0)
-      {
-	if (hdr)
-	{
-	  if ((hdr->content->length = atol (p)) < 0)
-	    hdr->content->length = -1;
-	}
-	matched = 1;
-      }
-      else if (ascii_strcasecmp (line+8, "description") == 0)
-      {
-	if (hdr)
-	{
-	  mutt_str_replace (&hdr->content->description, p);
-	  rfc2047_decode (&hdr->content->description);
-	}
-	matched = 1;
-      }
-      else if (ascii_strcasecmp (line+8, "disposition") == 0)
-      {
-	if (hdr)
-	  parse_content_disposition (p, hdr->content);
-	matched = 1;
-      }
-    }
-    break;
-    
+      break;
+
     case 'd':
-    if (!ascii_strcasecmp ("ate", line + 1))
-    {
-      mutt_str_replace (&e->date, p);
-      if (hdr)
-	hdr->date_sent = mutt_parse_date (p, hdr);
-      matched = 1;
-    }
-    break;
-    
-    case 'e':
-    if (!ascii_strcasecmp ("xpires", line + 1) &&
-	hdr && mutt_parse_date (p, NULL) < time (NULL))
-      hdr->expired = 1;
-    break;
-    
-    case 'f':
-    if (!ascii_strcasecmp ("rom", line + 1))
-    {
-      e->from = rfc822_parse_adrlist (e->from, p);
-      matched = 1;
-    }
-    break;
-    
-    case 'i':
-    if (!ascii_strcasecmp (line+1, "n-reply-to"))
-    {
-      mutt_free_list (&e->in_reply_to);
-      e->in_reply_to = mutt_parse_references (p, 1);
-      matched = 1;
-    }
-    break;
-    
-    case 'l':
-    if (!ascii_strcasecmp (line + 1, "ines"))
-    {
-      if (hdr)
+      if (!ascii_strcasecmp ("ate", line + 1))
       {
-	/* 
-	 * HACK - mutt has, for a very short time, produced negative
-	 * Lines header values.  Ignore them. 
-	 */
-	if (mutt_atoi (p, &hdr->lines) < 0 || hdr->lines < 0)
-	  hdr->lines = 0;
+        mutt_str_replace (&e->date, p);
+        if (hdr)
+          hdr->date_sent = mutt_parse_date (p, hdr);
+        matched = 1;
+      }
+      break;
+
+    case 'e':
+      if (!ascii_strcasecmp ("xpires", line + 1) &&
+          hdr && mutt_parse_date (p, NULL) < time (NULL))
+        hdr->expired = 1;
+      break;
+
+    case 'f':
+      if (!ascii_strcasecmp ("rom", line + 1))
+      {
+        e->from = rfc822_parse_adrlist (e->from, p);
+        matched = 1;
+      }
+      break;
+
+    case 'i':
+      if (!ascii_strcasecmp (line+1, "n-reply-to"))
+      {
+        mutt_free_list (&e->in_reply_to);
+        e->in_reply_to = mutt_parse_references (p, 1);
+        matched = 1;
+      }
+      break;
+
+    case 'l':
+      if (!ascii_strcasecmp (line + 1, "ines"))
+      {
+        if (hdr)
+        {
+          /*
+           * HACK - mutt has, for a very short time, produced negative
+           * Lines header values.  Ignore them.
+           */
+          if (mutt_atoi (p, &hdr->lines) < 0 || hdr->lines < 0)
+            hdr->lines = 0;
+        }
+
+        matched = 1;
+      }
+      else if (!ascii_strcasecmp (line + 1, "ist-Post"))
+      {
+        /* RFC 2369.  FIXME: We should ignore whitespace, but don't. */
+        if (strncmp (p, "NO", 2))
+        {
+          char *beg, *end;
+          for (beg = strchr (p, '<'); beg; beg = strchr (end, ','))
+          {
+            ++beg;
+            if (!(end = strchr (beg, '>')))
+              break;
+
+            /* Take the first mailto URL */
+            if (url_check_scheme (beg) == U_MAILTO)
+            {
+              FREE (&e->list_post);
+              e->list_post = mutt_substrdup (beg, end);
+              if (option (OPTAUTOSUBSCRIBE))
+                mutt_auto_subscribe (e->list_post);
+              break;
+            }
+          }
+        }
+        matched = 1;
+      }
+      break;
+
+    case 'm':
+      if (!ascii_strcasecmp (line + 1, "ime-version"))
+      {
+        if (hdr)
+          hdr->mime = 1;
+        matched = 1;
+      }
+      else if (!ascii_strcasecmp (line + 1, "essage-id"))
+      {
+        /* We add a new "Message-ID:" when building a message */
+        FREE (&e->message_id);
+        e->message_id = mutt_extract_message_id (p, NULL);
+        matched = 1;
+      }
+      else if (!ascii_strncasecmp (line + 1, "ail-", 4))
+      {
+        if (!ascii_strcasecmp (line + 5, "reply-to"))
+        {
+          /* override the Reply-To: field */
+          rfc822_free_address (&e->reply_to);
+          e->reply_to = rfc822_parse_adrlist (e->reply_to, p);
+          matched = 1;
+        }
+        else if (!ascii_strcasecmp (line + 5, "followup-to"))
+        {
+          e->mail_followup_to = rfc822_parse_adrlist (e->mail_followup_to, p);
+          matched = 1;
+        }
+      }
+      break;
+
+    case 'r':
+      if (!ascii_strcasecmp (line + 1, "eferences"))
+      {
+        mutt_free_list (&e->references);
+        e->references = mutt_parse_references (p, 0);
+        matched = 1;
+      }
+      else if (!ascii_strcasecmp (line + 1, "eply-to"))
+      {
+        e->reply_to = rfc822_parse_adrlist (e->reply_to, p);
+        matched = 1;
+      }
+      else if (!ascii_strcasecmp (line + 1, "eturn-path"))
+      {
+        e->return_path = rfc822_parse_adrlist (e->return_path, p);
+        matched = 1;
+      }
+      else if (!ascii_strcasecmp (line + 1, "eceived"))
+      {
+        if (hdr && !hdr->received)
+        {
+          char *d = strrchr (p, ';');
+
+          if (d)
+            hdr->received = mutt_parse_date (d + 1, NULL);
+        }
+      }
+      break;
+
+    case 's':
+      if (!ascii_strcasecmp (line + 1, "ubject"))
+      {
+        if (!e->subject)
+          e->subject = safe_strdup (p);
+        matched = 1;
+      }
+      else if (!ascii_strcasecmp (line + 1, "ender"))
+      {
+        e->sender = rfc822_parse_adrlist (e->sender, p);
+        matched = 1;
+      }
+      else if (!ascii_strcasecmp (line + 1, "tatus"))
+      {
+        if (hdr)
+        {
+          while (*p)
+          {
+            switch (*p)
+            {
+              case 'r':
+                hdr->replied = 1;
+                break;
+              case 'O':
+                hdr->old = 1;
+                break;
+              case 'R':
+                hdr->read = 1;
+                break;
+            }
+            p++;
+          }
+        }
+        matched = 1;
+      }
+      else if ((!ascii_strcasecmp ("upersedes", line + 1) ||
+                !ascii_strcasecmp ("upercedes", line + 1)) && hdr)
+      {
+        FREE(&e->supersedes);
+        e->supersedes = safe_strdup (p);
+      }
+      break;
+
+    case 't':
+      if (ascii_strcasecmp (line+1, "o") == 0)
+      {
+        e->to = rfc822_parse_adrlist (e->to, p);
+        matched = 1;
+      }
+      break;
+
+    case 'x':
+      if (ascii_strcasecmp (line+1, "-status") == 0)
+      {
+        if (hdr)
+        {
+          while (*p)
+          {
+            switch (*p)
+            {
+              case 'A':
+                hdr->replied = 1;
+                break;
+              case 'D':
+                hdr->deleted = 1;
+                break;
+              case 'F':
+                hdr->flagged = 1;
+                break;
+              default:
+                break;
+            }
+            p++;
+          }
+        }
+        matched = 1;
+      }
+      else if (ascii_strcasecmp (line+1, "-label") == 0)
+      {
+        FREE(&e->x_label);
+        e->x_label = safe_strdup(p);
+        matched = 1;
       }
 
-      matched = 1;
-    }
-    else if (!ascii_strcasecmp (line + 1, "ist-Post"))
-    {
-      /* RFC 2369.  FIXME: We should ignore whitespace, but don't. */
-      if (strncmp (p, "NO", 2))
-      {
-	char *beg, *end;
-	for (beg = strchr (p, '<'); beg; beg = strchr (end, ','))
-	{
-	  ++beg;
-	  if (!(end = strchr (beg, '>')))
-	    break;
-	  
-	  /* Take the first mailto URL */
-	  if (url_check_scheme (beg) == U_MAILTO)
-	  {
-	    FREE (&e->list_post);
-	    e->list_post = mutt_substrdup (beg, end);
-	    break;
-	  }
-	}
-      }
-      matched = 1;
-    }
-    break;
-    
-    case 'm':
-    if (!ascii_strcasecmp (line + 1, "ime-version"))
-    {
-      if (hdr)
-	hdr->mime = 1;
-      matched = 1;
-    }
-    else if (!ascii_strcasecmp (line + 1, "essage-id"))
-    {
-      /* We add a new "Message-ID:" when building a message */
-      FREE (&e->message_id);
-      e->message_id = mutt_extract_message_id (p, NULL);
-      matched = 1;
-    }
-    else if (!ascii_strncasecmp (line + 1, "ail-", 4))
-    {
-      if (!ascii_strcasecmp (line + 5, "reply-to"))
-      {
-	/* override the Reply-To: field */
-	rfc822_free_address (&e->reply_to);
-	e->reply_to = rfc822_parse_adrlist (e->reply_to, p);
-	matched = 1;
-      }
-      else if (!ascii_strcasecmp (line + 5, "followup-to"))
-      {
-	e->mail_followup_to = rfc822_parse_adrlist (e->mail_followup_to, p);
-	matched = 1;
-      }
-    }
-    break;
-    
-    case 'r':
-    if (!ascii_strcasecmp (line + 1, "eferences"))
-    {
-      mutt_free_list (&e->references);
-      e->references = mutt_parse_references (p, 0);
-      matched = 1;
-    }
-    else if (!ascii_strcasecmp (line + 1, "eply-to"))
-    {
-      e->reply_to = rfc822_parse_adrlist (e->reply_to, p);
-      matched = 1;
-    }
-    else if (!ascii_strcasecmp (line + 1, "eturn-path"))
-    {
-      e->return_path = rfc822_parse_adrlist (e->return_path, p);
-      matched = 1;
-    }
-    else if (!ascii_strcasecmp (line + 1, "eceived"))
-    {
-      if (hdr && !hdr->received)
-      {
-	char *d = strrchr (p, ';');
-	
-	if (d)
-	  hdr->received = mutt_parse_date (d + 1, NULL);
-      }
-    }
-    break;
-    
-    case 's':
-    if (!ascii_strcasecmp (line + 1, "ubject"))
-    {
-      if (!e->subject)
-	e->subject = safe_strdup (p);
-      matched = 1;
-    }
-    else if (!ascii_strcasecmp (line + 1, "ender"))
-    {
-      e->sender = rfc822_parse_adrlist (e->sender, p);
-      matched = 1;
-    }
-    else if (!ascii_strcasecmp (line + 1, "tatus"))
-    {
-      if (hdr)
-      {
-	while (*p)
-	{
-	  switch(*p)
-	  {
-	    case 'r':
-	    hdr->replied = 1;
-	    break;
-	    case 'O':
-	      hdr->old = 1;
-	    break;
-	    case 'R':
-	    hdr->read = 1;
-	    break;
-	  }
-	  p++;
-	}
-      }
-      matched = 1;
-    }
-    else if ((!ascii_strcasecmp ("upersedes", line + 1) ||
-	      !ascii_strcasecmp ("upercedes", line + 1)) && hdr)
-    {
-      FREE(&e->supersedes);
-      e->supersedes = safe_strdup (p);
-    }
-    break;
-    
-    case 't':
-    if (ascii_strcasecmp (line+1, "o") == 0)
-    {
-      e->to = rfc822_parse_adrlist (e->to, p);
-      matched = 1;
-    }
-    break;
-    
-    case 'x':
-    if (ascii_strcasecmp (line+1, "-status") == 0)
-    {
-      if (hdr)
-      {
-	while (*p)
-	{
-	  switch (*p)
-	  {
-	    case 'A':
-	    hdr->replied = 1;
-	    break;
-	    case 'D':
-	    hdr->deleted = 1;
-	    break;
-	    case 'F':
-	    hdr->flagged = 1;
-	    break;
-	    default:
-	    break;
-	  }
-	  p++;
-	}
-      }
-      matched = 1;
-    }
-    else if (ascii_strcasecmp (line+1, "-label") == 0)
-    {
-      FREE(&e->x_label);
-      e->x_label = safe_strdup(p);
-      matched = 1;
-    }
-    
     default:
-    break;
+      break;
   }
-  
+
   /* Keep track of the user-defined headers */
   if (!matched && user_hdrs)
   {
@@ -1302,11 +1353,11 @@ int mutt_parse_rfc822_line (ENVELOPE *e, HEADER *hdr, char *line, char *p, short
       *lastp = last;
   }
 
-  done:
+done:
   return matched;
 }
-  
-  
+
+
 /* mutt_read_rfc822_header() -- parses a RFC822 header
  *
  * Args:
@@ -1314,14 +1365,14 @@ int mutt_parse_rfc822_line (ENVELOPE *e, HEADER *hdr, char *line, char *p, short
  * f		stream to read from
  *
  * hdr		header structure of current message (optional).
- * 
+ *
  * user_hdrs	If set, store user headers.  Used for recall-message and
  * 		postpone modes.
- * 
+ *
  * weed		If this parameter is set and the user has activated the
  * 		$weed option, honor the header weed list for user headers.
  * 	        Used for recall-message.
- * 
+ *
  * Returns:     newly allocated envelope structure.  You should free it by
  *              mutt_free_envelope() when envelope stay unneeded.
  */
@@ -1354,7 +1405,7 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
   }
 
   while ((loc = ftello (f)),
-	  *(line = mutt_read_rfc822_line (f, line, &linelen)) != 0)
+         *(line = mutt_read_rfc822_line (f, line, &linelen)) != 0)
   {
     if ((p = strpbrk (line, ": \t")) == NULL || *p != ':')
     {
@@ -1396,8 +1447,7 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
 	  /* else overwrite */
 	  else
 	  {
-	    e->spam->dptr = e->spam->data;
-	    *e->spam->dptr = '\0';
+            mutt_buffer_clear (e->spam);
 	    mutt_buffer_addstr(e->spam, buf);
 	  }
 	}
@@ -1434,23 +1484,12 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
     hdr->content->hdr_offset = hdr->offset;
     hdr->content->offset = ftello (f);
 
-    /* do RFC2047 decoding */
-    rfc2047_decode_adrlist (e->from);
-    rfc2047_decode_adrlist (e->to);
-    rfc2047_decode_adrlist (e->cc);
-    rfc2047_decode_adrlist (e->bcc);
-    rfc2047_decode_adrlist (e->reply_to);
-    rfc2047_decode_adrlist (e->mail_followup_to);
-    rfc2047_decode_adrlist (e->return_path);
-    rfc2047_decode_adrlist (e->sender);
-    rfc2047_decode (&e->x_label);
+    rfc2047_decode_envelope (e);
 
     if (e->subject)
     {
       regmatch_t pmatch[1];
 
-      rfc2047_decode (&e->subject);
-
       if (regexec (ReplyRegexp.rx, e->subject, 1, pmatch, 0) == 0)
 	e->real_subj = e->subject + pmatch[0].rm_eo;
       else
@@ -1481,20 +1520,22 @@ ADDRESS *mutt_parse_adrlist (ADDRESS *p, const char *s)
   /* check for a simple whitespace separated list of addresses */
   if ((q = strpbrk (s, "\"<>():;,\\")) == NULL)
   {
-    char tmp[HUGE_STRING];
+    BUFFER *tmp;
     char *r;
 
-    strfcpy (tmp, s, sizeof (tmp));
-    r = tmp;
+    tmp = mutt_buffer_pool_get ();
+    mutt_buffer_strcpy (tmp, s);
+    r = tmp->data;
     while ((r = strtok (r, " \t")) != NULL)
     {
       p = rfc822_parse_adrlist (p, r);
       r = NULL;
     }
+    mutt_buffer_pool_release (&tmp);
   }
   else
     p = rfc822_parse_adrlist (p, s);
-  
+
   return p;
 }
 
@@ -1515,8 +1556,8 @@ static int count_body_parts_check(LIST **checklist, BODY *b, int dflt)
   {
     a = (ATTACH_MATCH *)type->data;
     dprint(5, (debugfile, "cbpc: %s %d/%s ?? %s/%s [%d]... ",
-		dflt ? "[OK]   " : "[EXCL] ",
-		b->type, b->subtype, a->major, a->minor, a->major_int));
+               dflt ? "[OK]   " : "[EXCL] ",
+               b->type, b->subtype, a->major, a->minor, a->major_int));
     if ((a->major_int == TYPEANY || a->major_int == b->type) &&
 	!regexec(&a->minor_rx, b->subtype, 0, NULL, 0))
     {
@@ -1551,10 +1592,10 @@ static int count_body_parts (BODY *body, int flags)
     shallrecurse = 0;
 
     dprint(5, (debugfile, "bp: desc=\"%s\"; fn=\"%s\", type=\"%d/%s\"\n",
-	   bp->description ? bp->description : ("none"),
-	   bp->filename ? bp->filename :
-			bp->d_filename ? bp->d_filename : "(none)",
-	   bp->type, bp->subtype ? bp->subtype : "*"));
+               bp->description ? bp->description : ("none"),
+               bp->filename ? bp->filename :
+               bp->d_filename ? bp->d_filename : "(none)",
+               bp->type, bp->subtype ? bp->subtype : "*"));
 
     if (bp->type == TYPEMESSAGE)
     {
@@ -1635,21 +1676,21 @@ int mutt_count_body_parts (CONTEXT *ctx, HEADER *hdr)
 
   if (hdr->attach_valid)
     return hdr->attach_total;
-  
+
   if (hdr->content->parts)
     keep_parts = 1;
   else
     mutt_parse_mime_message (ctx, hdr);
-  
+
   if (AttachAllow || AttachExclude || InlineAllow || InlineExclude)
     hdr->attach_total = count_body_parts(hdr->content, MUTT_PARTS_TOPLEVEL);
   else
     hdr->attach_total = 0;
 
   hdr->attach_valid = 1;
-  
+
   if (!keep_parts)
     mutt_free_body (&hdr->content->parts);
-  
+
   return hdr->attach_total;
 }
