@@ -1,20 +1,20 @@
 /*
  * Copyright (C) 1996-2000,2003,2013 Michael R. Elkins <me@mutt.org>
- * 
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */ 
+ */
 
 #if HAVE_CONFIG_H
 # include "config.h"
@@ -59,13 +59,13 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf);
 static ADDRESS *result_to_addr (QUERY *r)
 {
   static ADDRESS *tmp;
-  
+
   if (!(tmp = rfc822_cpy_adr (r->addr, 0)))
     return NULL;
-  
-  if(!tmp->next && !tmp->personal)
+
+  if (!tmp->next && !tmp->personal)
     tmp->personal = safe_strdup (r->name);
-  
+
   mutt_addrlist_to_intl (tmp, NULL);
   return tmp;
 }
@@ -94,7 +94,7 @@ static QUERY *run_query (char *s, int quiet)
   FILE *fp;
   QUERY *first = NULL;
   QUERY *cur = NULL;
-  char cmd[_POSIX_PATH_MAX];
+  BUFFER *cmd = NULL;
   char *buf = NULL;
   size_t buflen;
   int dummy = 0;
@@ -102,14 +102,17 @@ static QUERY *run_query (char *s, int quiet)
   char *p;
   pid_t thepid;
 
+  cmd = mutt_buffer_pool_get ();
+  mutt_expand_file_fmt (cmd, QueryCmd, s);
 
-  mutt_expand_file_fmt (cmd, sizeof(cmd), QueryCmd, s);
-
-  if ((thepid = mutt_create_filter (cmd, NULL, &fp, NULL)) < 0)
+  if ((thepid = mutt_create_filter (mutt_b2s (cmd), NULL, &fp, NULL)) < 0)
   {
-    dprint (1, (debugfile, "unable to fork command: %s", cmd));
+    dprint (1, (debugfile, "unable to fork command: %s", mutt_b2s (cmd)));
+    mutt_buffer_pool_release (&cmd);
     return 0;
   }
+  mutt_buffer_pool_release (&cmd);
+
   if (!quiet)
     mutt_message _("Waiting for response...");
   fgets (msg, sizeof (msg), fp);
@@ -153,7 +156,7 @@ static QUERY *run_query (char *s, int quiet)
     if (!quiet)
       mutt_message ("%s", msg);
   }
-  
+
   return first;
 }
 
@@ -167,7 +170,7 @@ static int query_search (MUTTMENU *m, regex_t *re, int n)
     return 0;
   if (table[n].data->addr)
   {
-    if (table[n].data->addr->personal && 
+    if (table[n].data->addr->personal &&
 	!regexec (re, table[n].data->addr->personal, 0, NULL, 0))
       return 0;
     if (table[n].data->addr->mailbox &&
@@ -179,7 +182,7 @@ static int query_search (MUTTMENU *m, regex_t *re, int n)
       return 0;
 #endif
   }
-  
+
   return REG_NOMATCH;
 }
 
@@ -197,31 +200,31 @@ static const char * query_format_str (char *dest, size_t destlen, size_t col, in
 
   switch (op)
   {
-  case 'a':
-    rfc822_write_address (buf2, sizeof (buf2), query->addr, 1);
-    mutt_format_s (dest, destlen, fmt, buf2);
-    break;
-  case 'c':
-    snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
-    snprintf (dest, destlen, tmp, query->num + 1);
-    break;
-  case 'e':
-    if (!optional)
-      mutt_format_s (dest, destlen, fmt, NONULL (query->other));
-    else if (!query->other || !*query->other)
-      optional = 0;
-    break;
-  case 'n':
-    mutt_format_s (dest, destlen, fmt, NONULL (query->name));
-    break;
-  case 't':
-    snprintf (tmp, sizeof (tmp), "%%%sc", fmt);
-    snprintf (dest, destlen, tmp, entry->tagged ? '*' : ' ');
-    break;
-  default:
-    snprintf (tmp, sizeof (tmp), "%%%sc", fmt);
-    snprintf (dest, destlen, tmp, op);
-    break;
+    case 'a':
+      rfc822_write_address (buf2, sizeof (buf2), query->addr, 1);
+      mutt_format_s (dest, destlen, fmt, buf2);
+      break;
+    case 'c':
+      snprintf (tmp, sizeof (tmp), "%%%sd", fmt);
+      snprintf (dest, destlen, tmp, query->num + 1);
+      break;
+    case 'e':
+      if (!optional)
+        mutt_format_s (dest, destlen, fmt, NONULL (query->other));
+      else if (!query->other || !*query->other)
+        optional = 0;
+      break;
+    case 'n':
+      mutt_format_s (dest, destlen, fmt, NONULL (query->name));
+      break;
+    case 't':
+      snprintf (tmp, sizeof (tmp), "%%%sc", fmt);
+      snprintf (dest, destlen, tmp, entry->tagged ? '*' : ' ');
+      break;
+    default:
+      snprintf (tmp, sizeof (tmp), "%%%sc", fmt);
+      snprintf (dest, destlen, tmp, op);
+      break;
   }
 
   if (optional)
@@ -245,7 +248,7 @@ static int query_tag (MUTTMENU *menu, int n, int m)
 {
   ENTRY *cur = &((ENTRY *) menu->data)[n];
   int ot = cur->tagged;
-  
+
   cur->tagged = m >= 0 ? m : !cur->tagged;
   return cur->tagged - ot;
 }
@@ -392,10 +395,10 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 
 	      if (op == OP_QUERY)
 	      {
-		menu->data = QueryTable = 
+		menu->data = QueryTable =
 		  (ENTRY *) safe_calloc (menu->max, sizeof (ENTRY));
 
-		for (i = 0, queryp = results; queryp; 
+		for (i = 0, queryp = results; queryp;
 		     queryp = queryp->next, i++)
 		  QueryTable[i].data = queryp;
 	      }
@@ -408,7 +411,7 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 
 		menu->data = QueryTable;
 
-		for (i = 0, queryp = results; queryp; 
+		for (i = 0, queryp = results; queryp;
 		     queryp = queryp->next, i++)
 		{
 		  /* once we hit new entries, clear/init the tag */
@@ -488,7 +491,7 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
       int tagged = 0;
       size_t curpos = 0;
 
-      memset (buf, 0, buflen); 
+      memset (buf, 0, buflen);
 
       /* check for tagged entries */
       for (i = 0; i < menu->max; i++)
@@ -524,7 +527,7 @@ static void query_menu (char *buf, size_t buflen, QUERY *results, int retbuf)
 	rfc822_write_address (buf, buflen, tmpa, 0);
 	rfc822_free_address (&tmpa);
       }
-      
+
     }
 
     free_query (&results);
