@@ -1,23 +1,24 @@
 /*
  * Copyright (C) 2005 Andreas Krennmair <ak@synflood.at>
  * Copyright (C) 2005 Peter J. Holzer <hjp@hjp.net>
- * Copyright (C) 2005-9 Rocco Rutte <pdmef@gmx.net>
- * 
+ * Copyright (C) 2005-2009 Rocco Rutte <pdmef@gmx.net>
+ * Copyright (C) 2010 Michael R. Elkins <me@mutt.org>
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
  *     (at your option) any later version.
- * 
+ *
  *     This program is distributed in the hope that it will be useful,
  *     but WITHOUT ANY WARRANTY; without even the implied warranty of
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
- * 
+ *
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- */ 
+ */
 
 /* This file was originally part of mutt-ng */
 
@@ -43,6 +44,7 @@ typedef struct flowed_state
 {
   size_t width;
   size_t spaces;
+  int delsp;
 } flowed_state_t;
 
 static int get_quote_level (const char *line)
@@ -59,7 +61,46 @@ static int get_quote_level (const char *line)
   return quoted;
 }
 
-static size_t print_indent (int ql, STATE *s, int sp)
+/* Determines whether to add spacing between/after each quote level:
+ *    >>>foo
+ * becomes
+ *    > > > foo
+ */
+static int space_quotes (STATE *s)
+{
+  /* Allow quote spacing in the pager even for OPTTEXTFLOWED,
+   * but obviously not when replying.
+   */
+  if (option (OPTTEXTFLOWED) && (s->flags & MUTT_REPLYING))
+    return 0;
+
+  return option (OPTREFLOWSPACEQUOTES);
+}
+
+/* Determines whether to add a trailing space to quotes:
+ *    >>> foo
+ * as opposed to
+ *    >>>foo
+ */
+static int add_quote_suffix (STATE *s, int ql)
+{
+  if (s->flags & MUTT_REPLYING)
+    return 0;
+
+  if (space_quotes (s))
+    return 0;
+
+  if (!ql && !s->prefix)
+    return 0;
+
+  /* The prefix will add its own space */
+  if (!option (OPTTEXTFLOWED) && !ql && s->prefix)
+    return 0;
+
+  return 1;
+}
+
+static size_t print_indent (int ql, STATE *s, int add_suffix)
 {
   int i;
   size_t wid = 0;
@@ -75,14 +116,21 @@ static size_t print_indent (int ql, STATE *s, int sp)
     {
       state_puts (s->prefix, s);
       wid = mutt_strwidth (s->prefix);
-      sp = 0;
     }
   }
   for (i = 0; i < ql; i++)
+  {
     state_putc ('>', s);
-  if (sp)
+    if (space_quotes (s) )
+      state_putc (' ', s);
+  }
+  if (add_suffix)
     state_putc (' ', s);
-  return ql + sp + wid;
+
+  if (space_quotes (s))
+    ql *= 2;
+
+  return ql + add_suffix + wid;
 }
 
 static void flush_par (STATE *s, flowed_state_t *fst)
@@ -95,13 +143,29 @@ static void flush_par (STATE *s, flowed_state_t *fst)
   fst->spaces = 0;
 }
 
+/* Calculate the paragraph width based upon the current quote level. The start
+ * of a quoted line will be ">>> ", so we need to subtract the space required
+ * for the prefix from the terminal width. */
 static int quote_width (STATE *s, int ql)
 {
-  size_t width = (Wrap ? mutt_term_width (Wrap) : FLOWED_MAX) - 1;
-  if (s->flags & M_REPLYING && width > FLOWED_MAX)
-    width = FLOWED_MAX;
-  if (ql + 1 < width)
-    width -= ql + 1;
+  int width = mutt_window_wrap_cols (MuttIndexWindow, ReflowWrap);
+  if (option(OPTTEXTFLOWED) && (s->flags & MUTT_REPLYING))
+  {
+    /* When replying, force a wrap at FLOWED_MAX to comply with RFC3676
+     * guidelines */
+    if (width > FLOWED_MAX)
+      width = FLOWED_MAX;
+    ++ql; /* When replying, we will add an additional quote level */
+  }
+  /* adjust the paragraph width subtracting the number of prefix chars */
+  width -= space_quotes (s) ? ql*2 : ql;
+  /* When displaying (not replying), there may be a space between the prefix
+   * string and the paragraph */
+  if (add_quote_suffix (s, ql))
+    --width;
+  /* failsafe for really long quotes */
+  if (width <= 0)
+    width = FLOWED_MAX; /* arbitrary, since the line will wrap */
   return width;
 }
 
@@ -110,6 +174,7 @@ static void print_flowed_line (char *line, STATE *s, int ql,
 {
   size_t width, w, words = 0;
   char *p;
+  char last;
 
   if (!line || !*line)
   {
@@ -121,6 +186,7 @@ static void print_flowed_line (char *line, STATE *s, int ql,
   }
 
   width = quote_width (s, ql);
+  last = line[mutt_strlen (line) - 1];
 
   dprint (4, (debugfile, "f=f: line [%s], width = %ld, spaces = %d\n",
 	      NONULL(line), (long)width, fst->spaces));
@@ -143,8 +209,12 @@ static void print_flowed_line (char *line, STATE *s, int ql,
 
     w = mutt_strwidth (p);
     /* see if we need to break the line but make sure the first
-       word is put on the line regardless */
-    if (w < width && w + fst->width + fst->spaces > width)
+       word is put on the line regardless;
+       if for DelSp=yes only one trailing space is used, we probably
+       have a long word that we should break within (we leave that
+       up to the pager or user) */
+    if (!(!fst->spaces && fst->delsp && last != ' ') &&
+	w < width && w + fst->width + fst->spaces > width)
     {
       dprint(4,(debugfile,"f=f: break line at %d, %d spaces left\n",
 		fst->width, fst->spaces));
@@ -159,8 +229,7 @@ static void print_flowed_line (char *line, STATE *s, int ql,
     }
 
     if (!words && !fst->width)
-      fst->width = print_indent (ql, s, !(s->flags & M_REPLYING) &&
-				 (ql > 0 || s->prefix));
+      fst->width = print_indent (ql, s, add_quote_suffix (s, ql));
     fst->width += w + fst->spaces;
     for ( ; fst->spaces; fst->spaces--)
       state_putc (' ', s);
@@ -175,7 +244,7 @@ static void print_flowed_line (char *line, STATE *s, int ql,
 static void print_fixed_line (const char *line, STATE *s, int ql,
 			      flowed_state_t *fst)
 {
-  print_indent (ql, s, !(s->flags & M_REPLYING) && (ql > 0 || s->prefix));
+  print_indent (ql, s, add_quote_suffix (s, ql));
   if (line && *line)
     state_puts (line, s);
   state_putc ('\n', s);
@@ -186,12 +255,10 @@ static void print_fixed_line (const char *line, STATE *s, int ql,
 
 int rfc3676_handler (BODY * a, STATE * s)
 {
-  int bytes = a->length;
-  char buf[LONG_STRING];
-  char *t = NULL;
+  char *buf = NULL, *t = NULL;
   unsigned int quotelevel = 0, newql = 0, sigsep = 0;
-  int buf_off = 0, buf_len;
-  int delsp = 0, fixed = 0;
+  int buf_off = 0, delsp = 0, fixed = 0;
+  size_t buf_len = 0, sz = 0;
   flowed_state_t fst;
 
   memset (&fst, 0, sizeof (fst));
@@ -201,16 +268,14 @@ int rfc3676_handler (BODY * a, STATE * s)
   {
     delsp = mutt_strlen (t) == 3 && ascii_strncasecmp (t, "yes", 3) == 0;
     t = NULL;
+    fst.delsp = 1;
   }
 
   dprint (4, (debugfile, "f=f: DelSp: %s\n", delsp ? "yes" : "no"));
 
-  while (bytes > 0 && fgets (buf, sizeof (buf), s->fpin))
+  while ((buf = mutt_read_line (buf, &sz, s->fpin, NULL, 0)))
   {
-
     buf_len = mutt_strlen (buf);
-    bytes -= buf_len;
-
     newql = get_quote_level (buf);
 
     /* end flowed paragraph (if we're within one) if quoting level
@@ -220,18 +285,6 @@ int rfc3676_handler (BODY * a, STATE * s)
       flush_par (s, &fst);
 
     quotelevel = newql;
-
-    /* XXX - If a line is longer than buf (shouldn't happen), it is split.
-     * This will almost always cause an unintended line break, and
-     * possibly a change in quoting level. But that's better than not
-     * displaying it at all.
-     */
-    if ((t = strrchr (buf, '\r')) || (t = strrchr (buf, '\n')))
-    {
-      *t = '\0';
-      buf_len = t - buf;
-    }
-
     buf_off = newql;
 
     /* respect sender's space-stuffing by removing one leading space */
@@ -264,6 +317,7 @@ int rfc3676_handler (BODY * a, STATE * s)
 
   flush_par (s, &fst);
 
+  FREE (&buf);
   return (0);
 }
 
@@ -300,7 +354,7 @@ void rfc3676_space_stuff (HEADER* hdr)
   if ((in = safe_fopen (hdr->content->filename, "r")) == NULL)
     return;
 
-  mutt_mktemp (tmpfile);
+  mutt_mktemp (tmpfile, sizeof (tmpfile));
   if ((out = safe_fopen (tmpfile, "w+")) == NULL)
   {
     safe_fclose (&in);
@@ -309,7 +363,8 @@ void rfc3676_space_stuff (HEADER* hdr)
 
   while (fgets (buf, sizeof (buf), in))
   {
-    if (ascii_strncmp ("From ", buf, 5) == 0 || buf[0] == ' ') {
+    if (ascii_strncmp ("From ", buf, 5) == 0 || buf[0] == ' ')
+    {
       fputc (' ', out);
 #if DEBUG
       lc++;
